@@ -1,3 +1,30 @@
+'''
+Touch Tracer Line Drawing Demonstration
+=======================================
+
+This demonstrates tracking each touch registered to a device. You should
+see a basic background image. When you press and hold the mouse, you
+should see cross-hairs with the coordinates written next to them. As
+you drag, it leaves a trail. Additional information, like pressure,
+will be shown if they are in your device's touch.profile.
+
+.. note::
+
+   A function `calculate_points` handling the points which will be drawn
+   has by default implemented a delay of 5 steps. To get more precise visual
+   results lower the value of the optional keyword argument `steps`.
+
+This program specifies an icon, the file icon.png, in its App subclass.
+It also uses the particle.png file as the source for drawing the trails which
+are white on transparent. The file touchtracer.kv describes the application.
+
+The file android.txt is used to package the application for use with the
+Kivy Launcher Android application. For Android devices, you can
+copy/paste this directory into /sdcard/kivy/touchtracer on your Android device.
+
+'''
+__version__ = '1.0'
+
 import kivy
 kivy.require('1.0.6')
 
@@ -17,7 +44,7 @@ def calculate_points(x1, y1, x2, y2, steps=5):
         return None
     o = []
     m = dist / steps
-    for i in xrange(1, int(m)):
+    for i in range(1, int(m)):
         mi = i / m
         lastx = x1 + dx * mi
         lasty = y1 + dy * mi
@@ -31,13 +58,19 @@ class Touchtracer(FloatLayout):
         win = self.get_parent_window()
         ud = touch.ud
         ud['group'] = g = str(touch.uid)
+        pointsize = 5
+        if 'pressure' in touch.profile:
+            ud['pressure'] = touch.pressure
+            pointsize = (touch.pressure * 100000) ** 2
+        ud['color'] = random()
+
         with self.canvas:
-            ud['color'] = Color(random(), 1, 1, mode='hsv', group=g)
-            ud['lines'] = (
+            Color(ud['color'], 1, 1, mode='hsv', group=g)
+            ud['lines'] = [
                 Rectangle(pos=(touch.x, 0), size=(1, win.height), group=g),
                 Rectangle(pos=(0, touch.y), size=(win.width, 1), group=g),
                 Point(points=(touch.x, touch.y), source='particle.png',
-                      pointsize=5, group=g))
+                      pointsize=pointsize, group=g)]
 
         ud['label'] = Label(size_hint=(None, None))
         self.update_touch_label(ud['label'], touch)
@@ -52,14 +85,34 @@ class Touchtracer(FloatLayout):
         ud['lines'][0].pos = touch.x, 0
         ud['lines'][1].pos = 0, touch.y
 
-        points = ud['lines'][2].points
-        oldx, oldy = points[-2], points[-1]
+        index = -1
+
+        while True:
+            try:
+                points = ud['lines'][index].points
+                oldx, oldy = points[-2], points[-1]
+                break
+            except:
+                index -= 1
+
         points = calculate_points(oldx, oldy, touch.x, touch.y)
+
+        # if pressure changed create a new point instruction
+        if 'pressure' in ud:
+            if not .95 < (touch.pressure / ud['pressure']) < 1.05:
+                g = ud['group']
+                pointsize = (touch.pressure * 100000) ** 2
+                with self.canvas:
+                    Color(ud['color'], 1, 1, mode='hsv', group=g)
+                    ud['lines'].append(
+                        Point(points=(), source='particle.png',
+                              pointsize=pointsize, group=g))
+
         if points:
             try:
-                lp = ud['lines'][2].add_point
-                for idx in xrange(0, len(points), 2):
-                    lp(points[idx], points[idx+1])
+                lp = ud['lines'][-1].add_point
+                for idx in range(0, len(points), 2):
+                    lp(points[idx], points[idx + 1])
             except GraphicException:
                 pass
 
@@ -98,5 +151,6 @@ class TouchtracerApp(App):
     def on_pause(self):
         return True
 
-if __name__ in ('__main__', '__android__'):
+
+if __name__ == '__main__':
     TouchtracerApp().run()
