@@ -3,11 +3,11 @@ Tree shader
 ===========
 
 This example is an experimentation to show how we can use shader for a tree
-subset. Here, we made a ShaderTreeWidget, different than the ShaderWidget in the
-plasma.py example.
+subset. Here, we made a ShaderTreeWidget, different than the ShaderWidget
+in the plasma.py example.
 
-The ShaderTree widget create a Frambuffer, render his children on it, and render
-the Framebuffer with a specific Shader.
+The ShaderTree widget create a Frambuffer, render his children on it, and
+render the Framebuffer with a specific Shader.
 With this way, you can apply cool effect on your widgets :)
 
 '''
@@ -97,6 +97,7 @@ void main() {
 }
 '''
 
+
 class ShaderWidget(FloatLayout):
 
     # property to set the source code for fragment shader
@@ -108,12 +109,12 @@ class ShaderWidget(FloatLayout):
     def __init__(self, **kwargs):
         # Instead of using canvas, we will use a RenderContext,
         # and change the default shader used.
-        self.canvas = RenderContext()
+        self.canvas = RenderContext(use_parent_projection=True)
 
         # We create a framebuffer at the size of the window
         # FIXME: this should be created at the size of the widget
         with self.canvas:
-            self.fbo = Fbo(size=Window.size)
+            self.fbo = Fbo(size=Window.size, use_parent_projection=True)
 
         # Set the fbo background to black.
         with self.fbo:
@@ -127,15 +128,13 @@ class ShaderWidget(FloatLayout):
         # We'll update our glsl variables in a clock
         Clock.schedule_interval(self.update_glsl, 0)
 
-        # Don't forget to set the texture property to the texture of framebuffer
+        # Don't forget to set the texture property to the texture
+        # of framebuffer
         self.texture = self.fbo.texture
 
     def update_glsl(self, *largs):
         self.canvas['time'] = Clock.get_boottime()
-        self.canvas['resolution'] = map(float, self.size)
-        # This is needed for the default vertex shader.
-        self.fbo['projection_mat'] = Window.render_context['projection_mat']
-        self.canvas['projection_mat'] = Window.render_context['projection_mat']
+        self.canvas['resolution'] = [float(v) for v in self.size]
 
     def on_fs(self, instance, value):
         # set the fragment shader to our source code
@@ -194,12 +193,15 @@ class ShaderTreeApp(App):
         # shader
         btn = Button(text='Change fragment shader', size_hint=(1, None),
                      height=50)
+
         def change(*largs):
             sw.fs = available_shaders[self.shader_index]
-            self.shader_index = (self.shader_index + 1) % len(available_shaders)
+            self.shader_index = ((self.shader_index + 1) %
+                                 len(available_shaders))
         btn.bind(on_release=change)
         root.add_widget(btn)
         return root
 
+
 if __name__ == '__main__':
     ShaderTreeApp().run()
