@@ -3,8 +3,8 @@ Animation
 =========
 
 :class:`Animation` and :class:`AnimationTransition` are used to animate
-:class:`~kivy.uix.widget.Widget` properties. You must specify (minimum) a
-property name and target value. To use Animation, follow these steps:
+:class:`~kivy.uix.widget.Widget` properties. You must specify at least a
+property name and target value. To use an Animation, follow these steps:
 
     * Setup an Animation object
     * Use the Animation object on a Widget
@@ -18,7 +18,7 @@ where you want the widget positioned at the end of the animation::
     anim = Animation(x=100, y=100)
     anim.start(widget)
 
-The animation will last for 1 second unless :data:`duration` is specified.
+The animation will last for 1 second unless :attr:`duration` is specified.
 When anim.start() is called, the Widget will move smoothly from the current
 x/y position to (100, 100).
 
@@ -26,20 +26,20 @@ Multiple properties and transitions
 -----------------------------------
 
 You can animate multiple properties and use built-in or custom transition
-functions using :data:`transition` (or `t=` shortcut). For example,
+functions using :attr:`transition` (or the `t=` shortcut). For example,
 to animate the position and size using the 'in_quad' transition::
 
     anim = Animation(x=50, size=(80, 80), t='in_quad')
     anim.start(widget)
 
 Note that the `t=` parameter can be the string name of a method in the
-:class:`AnimationTransition` class, or your own animation function.
+:class:`AnimationTransition` class or your own animation function.
 
 Sequential animation
 --------------------
 
 To join animations sequentially, use the '+' operator. The following example
-will animate to x=50 over 1 second, then animate size to (80, 80) over the
+will animate to x=50 over 1 second, then animate the size to (80, 80) over the
 next two seconds::
 
     anim = Animation(x=50) + Animation(size=(80, 80), duration=2.)
@@ -49,82 +49,111 @@ Parallel animation
 ------------------
 
 To join animations in parallel, use the '&' operator. The following example
-will animate position to (80, 10) over 1 second, while in parallel animating
-the first half of size=(800, 800)::
+will animate the position to (80, 10) over 1 second, whilst in parallel
+animating the size to (800, 800)::
 
     anim = Animation(pos=(80, 10))
     anim &= Animation(size=(800, 800), duration=2.)
     anim.start(widget)
 
+Keep in mind that creating overlapping animations on the same property may have
+unexpected results. If you want to apply multiple animations to the same
+property, you should either schedule them sequentially (via the '+' operator or
+using the *on_complete* callback) or cancel previous animations using the
+:attr:`~Animation.cancel_all` method.
+
+Repeating animation
+-------------------
+
+.. versionadded:: 1.8.0
+
+.. note::
+    This is currently only implemented for 'Sequence' animations.
+
+To set an animation to repeat, simply set the :attr:`Sequence.repeat`
+property to `True`::
+
+    anim = Animation(...) + Animation(...)
+    anim.repeat = True
+    anim.start(widget)
+
+For flow control of animations such as stopping and cancelling, use the methods
+already in place in the animation module.
 '''
 
 __all__ = ('Animation', 'AnimationTransition')
 
-from types import ListType, TupleType, DictType
 from math import sqrt, cos, sin, pi
 from kivy.event import EventDispatcher
 from kivy.clock import Clock
+from kivy.compat import string_types, iterkeys
+from kivy.weakproxy import WeakProxy
 
 
 class Animation(EventDispatcher):
-    '''Create an animation definition that can be used to animate a Widget
+    '''Create an animation definition that can be used to animate a Widget.
 
     :Parameters:
-        `duration` or `d`: float, default to 1.
-            Duration of the animation, in seconds
+        `duration` or `d`: float, defaults to 1.
+            Duration of the animation, in seconds.
         `transition` or `t`: str or func
             Transition function for animate properties. It can be the name of a
-            method from :class:`AnimationTransition`
+            method from :class:`AnimationTransition`.
         `step` or `s`: float
-            Step in milliseconds of the animation. Default to 1 / 60.
+            Step in milliseconds of the animation. Defaults to 0, which means
+            the animation is updated for every frame.
+
+            To update the animation less often, set the step value to a float.
+            For example, if you want to animate at 30 FPS, use s=1/30.
 
     :Events:
-        `on_start`: widget
-            Fired when the animation is started on a widget
-        `on_complete`: widget
-            Fired when the animation is completed or stopped on a widget
-        `on_progress`: widget, progression
-            Fired when the progression of the animation is changing
+        `on_start`: animation, widget
+            Fired when the animation is started on a widget.
+        `on_complete`: animation, widget
+            Fired when the animation is completed or stopped on a widget.
+        `on_progress`: animation, widget, progression
+            Fired when the progression of the animation is changing.
 
     .. versionchanged:: 1.4.0
         Added s/step parameter.
 
+    .. versionchanged:: 1.10.0
+        The default value of the step parameter was changed from 1/60. to 0.
     '''
 
+    _update_ev = None
+
     _instances = set()
 
     __events__ = ('on_start', 'on_progress', 'on_complete')
 
     def __init__(self, **kw):
-        super(Animation, self).__init__(**kw)
-
+        super(Animation, self).__init__()
         # Initialize
         self._clock_installed = False
-        self._duration = kw.get('d', kw.get('duration', 1.))
-        self._transition = kw.get('t', kw.get('transition', 'linear'))
-        self._step = kw.get('s', kw.get('step', 1. / 60.))
-        if isinstance(self._transition, basestring):
+        self._duration = kw.pop('d', kw.pop('duration', 1.))
+        self._transition = kw.pop('t', kw.pop('transition', 'linear'))
+        self._step = kw.pop('s', kw.pop('step', 0))
+        if isinstance(self._transition, string_types):
             self._transition = getattr(AnimationTransition, self._transition)
-        for key in ('d', 't', 's', 'step', 'duration', 'transition'):
-            kw.pop(key, None)
         self._animated_properties = kw
         self._widgets = {}
 
     @property
     def duration(self):
-        '''Return the duration of the animation
+        '''Return the duration of the animation.
         '''
         return self._duration
 
     @property
     def transition(self):
-        '''Return the transition of the animation
+        '''Return the transition of the animation.
         '''
         return self._transition
 
     @property
     def animated_properties(self):
-        '''Return the properties used to animate
+        '''Return the properties used to animate.
         '''
         return self._animated_properties
 
@@ -152,7 +181,7 @@ class Animation(EventDispatcher):
     @staticmethod
     def cancel_all(widget, *largs):
         '''Cancel all animations that concern a specific widget / list of
-        properties. see :data:`cancel`
+        properties. See :attr:`cancel`.
 
         Example::
 
@@ -173,7 +202,7 @@ class Animation(EventDispatcher):
                 animation.cancel(widget)
 
     def start(self, widget):
-        '''Start the animation on a widget
+        '''Start the animation on a widget.
         '''
         self.stop(widget)
         self._initialize(widget)
@@ -181,31 +210,31 @@ class Animation(EventDispatcher):
         self.dispatch('on_start', widget)
 
     def stop(self, widget):
-        '''Stop the animation previously applied on a widget, triggering
-        `on_complete` event '''
-        props = self._widgets.pop(widget, None)
+        '''Stop the animation previously applied to a widget, triggering the
+        `on_complete` event.'''
+        props = self._widgets.pop(widget.uid, None)
         if props:
             self.dispatch('on_complete', widget)
         self.cancel(widget)
 
     def cancel(self, widget):
-        '''Cancel the animation previously applied on a widget. Same
-        effect as :data:`stop`, except the `on_complete` event will
+        '''Cancel the animation previously applied to a widget. Same
+        effect as :attr:`stop`, except the `on_complete` event will
         *not* be triggered!
 
         .. versionadded:: 1.4.0
         '''
-        self._widgets.pop(widget, None)
+        self._widgets.pop(widget.uid, None)
         self._clock_uninstall()
         if not self._widgets:
             self._unregister()
 
     def stop_property(self, widget, prop):
         '''Even if an animation is running, remove a property. It will not be
-        animated further. If it was the only/last property being animated on.
-        the widget, the animation will be stopped (see :data:`stop`)
+        animated futher. If it was the only/last property being animated,
+        the animation will be stopped (see :attr:`stop`).
         '''
-        props = self._widgets.get(widget, None)
+        props = self._widgets.get(widget.uid, None)
         if not props:
             return
         props['properties'].pop(prop, None)
@@ -216,12 +245,12 @@ class Animation(EventDispatcher):
 
     def cancel_property(self, widget, prop):
         '''Even if an animation is running, remove a property. It will not be
-        animated further. If it was the only/last property being animated on.
-        the widget, the animation will be canceled (see :data:`cancel`)
+        animated further. If it was the only/last property being animated,
+        the animation will be canceled (see :attr:`cancel`)
 
         .. versionadded:: 1.4.0
         '''
-        props = self._widgets.get(widget, None)
+        props = self._widgets.get(widget.uid, None)
         if not props:
             return
         props['properties'].pop(prop, None)
@@ -230,6 +259,15 @@ class Animation(EventDispatcher):
         if not props['properties']:
             self.cancel(widget)
 
+    def have_properties_to_animate(self, widget):
+        '''Return True if a widget still has properties to animate.
+
+        .. versionadded:: 1.8.0
+        '''
+        props = self._widgets.get(widget.uid, None)
+        if props and props['properties']:
+            return True
+
     #
     # Private
     #
@@ -241,14 +279,20 @@ class Animation(EventDispatcher):
             Animation._instances.remove(self)
 
     def _initialize(self, widget):
-        d = self._widgets[widget] = {
+        d = self._widgets[widget.uid] = {
+            'widget': widget,
             'properties': {},
             'time': None}
 
         # get current values
         p = d['properties']
-        for key, value in self._animated_properties.iteritems():
-            p[key] = (getattr(widget, key), value)
+        for key, value in self._animated_properties.items():
+            original_value = getattr(widget, key)
+            if isinstance(original_value, (tuple, list)):
+                original_value = original_value[:]
+            elif isinstance(original_value, dict):
+                original_value = original_value.copy()
+            p[key] = (original_value, value)
 
         # install clock
         self._clock_install()
@@ -256,32 +300,47 @@ class Animation(EventDispatcher):
     def _clock_install(self):
         if self._clock_installed:
             return
-        Clock.schedule_interval(self._update, self._step)
+        self._update_ev = Clock.schedule_interval(self._update, self._step)
         self._clock_installed = True
 
     def _clock_uninstall(self):
         if self._widgets or not self._clock_installed:
             return
         self._clock_installed = False
-        Clock.unschedule(self._update)
+        if self._update_ev is not None:
+            self._update_ev.cancel()
+            self._update_ev = None
 
     def _update(self, dt):
         widgets = self._widgets
         transition = self._transition
         calculate = self._calculate
-        for widget in widgets.keys()[:]:
-            anim = widgets[widget]
+        for uid in list(widgets.keys())[:]:
+            anim = widgets[uid]
+            widget = anim['widget']
+
+            if isinstance(widget, WeakProxy) and not len(dir(widget)):
+                # empty proxy, widget is gone. ref: #2458
+                self._widgets.pop(uid, None)
+                self._clock_uninstall()
+                if not self._widgets:
+                    self._unregister()
+                continue
+
             if anim['time'] is None:
                 anim['time'] = 0.
             else:
                 anim['time'] += dt
 
             # calculate progression
-            progress = min(1., anim['time'] / self._duration)
+            if self._duration:
+                progress = min(1., anim['time'] / self._duration)
+            else:
+                progress = 1
             t = transition(progress)
 
             # apply progression on widget
-            for key, values in anim['properties'].iteritems():
+            for key, values in anim['properties'].items():
                 a, b = values
                 value = calculate(a, b, t)
                 setattr(widget, key, value)
@@ -294,16 +353,16 @@ class Animation(EventDispatcher):
 
     def _calculate(self, a, b, t):
         _calculate = self._calculate
-        if isinstance(a, ListType) or isinstance(a, TupleType):
-            if isinstance(a, ListType):
+        if isinstance(a, list) or isinstance(a, tuple):
+            if isinstance(a, list):
                 tp = list
             else:
                 tp = tuple
-            return tp([_calculate(a[x], b[x], t) for x in xrange(len(a))])
-        elif isinstance(a, DictType):
+            return tp([_calculate(a[x], b[x], t) for x in range(len(a))])
+        elif isinstance(a, dict):
             d = {}
-            for x in a.iterkeys():
-                if not x in b.keys():
+            for x in iterkeys(a):
+                if x not in b:
                     # User requested to animate only part of the dict.
                     # Copy the rest
                     d[x] = a[x]
@@ -336,11 +395,15 @@ class Sequence(Animation):
 
     def __init__(self, anim1, anim2):
         super(Sequence, self).__init__()
+
+        #: Repeat the sequence. See 'Repeating animation' in the header
+        #: documentation.
+        self.repeat = False
+
         self.anim1 = anim1
         self.anim2 = anim2
 
         self.anim1.bind(on_start=self.on_anim1_start,
-                        on_complete=self.on_anim1_complete,
                         on_progress=self.on_anim1_progress)
         self.anim2.bind(on_complete=self.on_anim2_complete,
                         on_progress=self.on_anim2_progress)
@@ -351,28 +414,68 @@ class Sequence(Animation):
 
     def start(self, widget):
         self.stop(widget)
-        self.anim1.start(widget)
-        self._widgets[widget] = True
+        self._widgets[widget.uid] = True
         self._register()
+        self.anim1.start(widget)
+        self.anim1.bind(on_complete=self.on_anim1_complete)
 
     def stop(self, widget):
         self.anim1.stop(widget)
         self.anim2.stop(widget)
-        self._widgets.pop(widget, None)
-        if not self._widgets:
-            self._unregister()
+        props = self._widgets.pop(widget.uid, None)
+        if props:
+            self.dispatch('on_complete', widget)
+        super(Sequence, self).cancel(widget)
+
+    def stop_property(self, widget, prop):
+        self.anim1.stop_property(widget, prop)
+        self.anim2.stop_property(widget, prop)
+        if (not self.anim1.have_properties_to_animate(widget) and
+                not self.anim2.have_properties_to_animate(widget)):
+            self.stop(widget)
+
+    def cancel(self, widget):
+        self.anim1.cancel(widget)
+        self.anim2.cancel(widget)
+        super(Sequence, self).cancel(widget)
+
+    def cancel_property(self, widget, prop):
+        '''Even if an animation is running, remove a property. It will not be
+        animated further. If it was the only/last property being animated,
+        the animation will be canceled (see :attr:`cancel`)
+
+        This method overrides `:class:kivy.animation.Animation`'s
+        version, to cancel it on all animations of the Sequence.
+
+        .. versionadded:: 1.10.0
+        '''
+        self.anim1.cancel_property(widget, prop)
+        self.anim2.cancel_property(widget, prop)
+        if (not self.anim1.have_properties_to_animate(widget) and
+                not self.anim2.have_properties_to_animate(widget)):
+            self.cancel(widget)
 
     def on_anim1_start(self, instance, widget):
         self.dispatch('on_start', widget)
 
     def on_anim1_complete(self, instance, widget):
+        self.anim1.unbind(on_complete=self.on_anim1_complete)
         self.anim2.start(widget)
 
     def on_anim1_progress(self, instance, widget, progress):
         self.dispatch('on_progress', widget, progress / 2.)
 
     def on_anim2_complete(self, instance, widget):
-        self.dispatch('on_complete', widget)
+        '''Repeating logic used with boolean variable "repeat".
+
+        .. versionadded:: 1.7.1
+        '''
+        if self.repeat:
+            self.anim1.start(widget)
+            self.anim1.bind(on_complete=self.on_anim1_complete)
+        else:
+            self.dispatch('on_complete', widget)
+            self.cancel(widget)
 
     def on_anim2_progress(self, instance, widget, progress):
         self.dispatch('on_progress', widget, .5 + progress / 2.)
@@ -396,29 +499,42 @@ class Parallel(Animation):
         self.stop(widget)
         self.anim1.start(widget)
         self.anim2.start(widget)
-        self._widgets[widget] = {'complete': 0}
+        self._widgets[widget.uid] = {'complete': 0}
         self._register()
         self.dispatch('on_start', widget)
 
     def stop(self, widget):
         self.anim1.stop(widget)
         self.anim2.stop(widget)
-        self._widgets.pop(widget, None)
-        if not self._widgets:
-            self._unregister()
+        props = self._widgets.pop(widget.uid, None)
+        if props:
+            self.dispatch('on_complete', widget)
+        super(Parallel, self).cancel(widget)
+
+    def stop_property(self, widget, prop):
+        self.anim1.stop_property(widget, prop)
+        self.anim2.stop_property(widget, prop)
+        if (not self.anim1.have_properties_to_animate(widget) and
+                not self.anim2.have_properties_to_animate(widget)):
+            self.stop(widget)
+
+    def cancel(self, widget):
+        self.anim1.cancel(widget)
+        self.anim2.cancel(widget)
+        super(Parallel, self).cancel(widget)
 
     def on_anim_complete(self, instance, widget):
-        self._widgets[widget]['complete'] += 1
-        if self._widgets[widget]['complete'] == 2:
-            self.dispatch('on_complete', widget)
+        self._widgets[widget.uid]['complete'] += 1
+        if self._widgets[widget.uid]['complete'] == 2:
+            self.stop(widget)
 
 
 class AnimationTransition(object):
-    '''Collection of animation function, to be used with Animation object.
-    Easing Functions ported into Kivy from Clutter Project
-    http://www.clutter-project.org/docs/clutter/stable/ClutterAlpha.html
+    '''Collection of animation functions to be used with the Animation object.
+    Easing Functions ported to Kivy from the Clutter Project
+    https://developer.gnome.org/clutter/stable/ClutterAlpha.html
 
-    `progress` parameter in each animation functions is between 0-1 range.
+    The `progress` parameter in each animation function is in the range 0-1.
     '''
 
     @staticmethod
