@@ -2,8 +2,16 @@
 Video
 =====
 
-Core class for reading video file and manage the
-:class:`kivy.graphics.texture.Texture` video.
+Core class for reading video files and managing the video
+:class:`~kivy.graphics.texture.Texture`.
+
+.. versionchanged:: 1.10.0
+    The pyglet, pygst and gi providers have been removed.
+
+.. versionchanged:: 1.8.0
+    There are now 2 distinct Gstreamer implementations: one using Gi/Gst
+    working for both Python 2+3 with Gstreamer 1.0, and one using PyGST
+    working only for Python 2 + Gstreamer 0.10.
 
 .. note::
 
@@ -15,45 +23,49 @@ __all__ = ('VideoBase', 'Video')
 from kivy.clock import Clock
 from kivy.core import core_select_lib
 from kivy.event import EventDispatcher
+from kivy.logger import Logger
+from kivy.compat import PY2
 
 
 class VideoBase(EventDispatcher):
-    '''VideoBase, a class to implement a video reader.
+    '''VideoBase, a class used to implement a video reader.
 
     :Parameters:
-        `filename` : str
+        `filename`: str
             Filename of the video. Can be a file or an URI.
-        `eos` : str, default to 'pause'
-            Action to do when EOS is hit. Can be one of 'pause', 'stop' or
-            'loop'
-            .. versionchanged:: added 'pause'
-        `async` : bool, default to True
-            Asynchronous loading (may be not supported by all providers)
-        `autoplay` : bool, default to False
-            Auto play the video at init
+        `eos`: str, defaults to 'pause'
+            Action to take when EOS is hit. Can be one of 'pause', 'stop' or
+            'loop'.
+
+            .. versionchanged:: unknown
+                added 'pause'
+
+        `async`: bool, defaults to True
+            Load the video asynchronously (may be not supported by all
+            providers).
+        `autoplay`: bool, defaults to False
+            Auto play the video on init.
 
     :Events:
         `on_eos`
-            Fired when EOS is hit
+            Fired when EOS is hit.
         `on_load`
-            Fired when the video is loaded, texture is available
+            Fired when the video is loaded and the texture is available.
         `on_frame`
-            Fired when a new frame is written on texture
+            Fired when a new frame is written to the texture.
     '''
 
     __slots__ = ('_wantplay', '_buffer', '_filename', '_texture',
                  '_volume', 'eos', '_state', '_async', '_autoplay')
 
+    __events__ = ('on_eos', 'on_load', 'on_frame')
+
     def __init__(self, **kwargs):
         kwargs.setdefault('filename', None)
         kwargs.setdefault('eos', 'stop')
         kwargs.setdefault('async', True)
         kwargs.setdefault('autoplay', False)
 
-        self.register_event_type('on_eos')
-        self.register_event_type('on_load')
-        self.register_event_type('on_frame')
-
         super(VideoBase, self).__init__()
 
         self._wantplay = False
@@ -101,8 +113,8 @@ class VideoBase(EventDispatcher):
         self.load()
 
     filename = property(lambda self: self._get_filename(),
-            lambda self, x: self._set_filename(x),
-            doc='Get/set the filename/uri of current video')
+                        lambda self, x: self._set_filename(x),
+                        doc='Get/set the filename/uri of the current video')
 
     def _get_position(self):
         return 0
@@ -111,8 +123,8 @@ class VideoBase(EventDispatcher):
         self.seek(pos)
 
     position = property(lambda self: self._get_position(),
-            lambda self, x: self._set_position(x),
-            doc='Get/set the position in the video (in seconds)')
+                        lambda self, x: self._set_position(x),
+                        doc='Get/set the position in the video (in seconds)')
 
     def _get_volume(self):
         return self._volume
@@ -121,42 +133,43 @@ class VideoBase(EventDispatcher):
         self._volume = volume
 
     volume = property(lambda self: self._get_volume(),
-            lambda self, x: self._set_volume(x),
-            doc='Get/set the volume in the video (1.0 = 100%)')
+                      lambda self, x: self._set_volume(x),
+                      doc='Get/set the volume in the video (1.0 = 100%)')
 
     def _get_duration(self):
         return 0
 
     duration = property(lambda self: self._get_duration(),
-            doc='Get the video duration (in seconds)')
+                        doc='Get the video duration (in seconds)')
 
     def _get_texture(self):
         return self._texture
 
     texture = property(lambda self: self._get_texture(),
-            doc='Get the video texture')
+                       doc='Get the video texture')
 
     def _get_state(self):
         return self._state
 
     state = property(lambda self: self._get_state(),
-            doc='Get the video playing status')
+                     doc='Get the video playing status')
 
-    def _do_eos(self):
-        '''.. versionchanged:: 1.4.0
-        Now dispatches the `on_eos` event.
+    def _do_eos(self, *args):
+        '''
+        .. versionchanged:: 1.4.0
+            Now dispatches the `on_eos` event.
         '''
         if self.eos == 'pause':
             self.pause()
         elif self.eos == 'stop':
             self.stop()
         elif self.eos == 'loop':
-            self.stop()
+            self.position = 0
             self.play()
 
         self.dispatch('on_eos')
 
-    def _update(self):
+    def _update(self, dt):
         '''Update the video content to texture.
         '''
         pass
@@ -171,6 +184,7 @@ class VideoBase(EventDispatcher):
 
     def pause(self):
         '''Pause the video
+
         .. versionadded:: 1.4.0
         '''
         self._state = 'paused'
@@ -189,9 +203,16 @@ class VideoBase(EventDispatcher):
 
 
 # Load the appropriate provider
-Video = core_select_lib('video', (
-    ('gstreamer', 'video_gstreamer', 'VideoGStreamer'),
+video_providers = []
+try:
+    from kivy.lib.gstplayer import GstPlayer  # NOQA
+    video_providers += [('gstplayer', 'video_gstplayer', 'VideoGstplayer')]
+except ImportError:
+    pass
+video_providers += [
     ('ffmpeg', 'video_ffmpeg', 'VideoFFMpeg'),
-    ('pyglet', 'video_pyglet', 'VideoPyglet'),
-))
+    ('ffpyplayer', 'video_ffpyplayer', 'VideoFFPy'),
+    ('null', 'video_null', 'VideoNull')]
 
+
+Video = core_select_lib('video', video_providers)
