@@ -4,10 +4,10 @@ Auto Create Input Provider Config Entry for Available MT Hardware (linux only).
 
 Thanks to Marc Tardif for the probing code, taken from scan-for-mt-device.
 
-The device discovery is done by this provider. However, the reading of input can
-be made by other providers like: hidinput, mtdev, linuxwacom. mtdev is used
-prior to other providers. For more information about mtdev, check
-:py:class:`~kivy.input.providers.mtdev`.
+The device discovery is done by this provider. However, the reading of
+input can be performed by other providers like: hidinput, mtdev and
+linuxwacom. mtdev is used prior to other providers. For more
+information about mtdev, check :py:class:`~kivy.input.providers.mtdev`.
 
 Here is an example of auto creation::
 
@@ -30,9 +30,9 @@ Here is an example of auto creation::
     pen = probesysfs,match=E3 Pen,provider=linuxwacom,
         select_all=1,param=mode=pen
 
-By default, ProbeSysfs module will enumerate hardware from /sys/class/input
+By default, ProbeSysfs module will enumerate hardware from the /sys/class/input
 device, and configure hardware with ABS_MT_POSITION_X capability. But for
-example, wacom screen doesn't support this capability. You can prevent this
+example, the wacom screen doesn't support this capability. You can prevent this
 behavior by putting select_all=1 in your config line.
 '''
 
@@ -51,16 +51,22 @@ else:
     from subprocess import Popen, PIPE
     from kivy.logger import Logger
     from kivy.input.provider import MotionEventProvider
+    from kivy.input.providers.mouse import MouseMotionEventProvider
     from kivy.input.factory import MotionEventFactory
+    from kivy.config import _is_rpi
+
+    EventLoop = None
 
     # See linux/input.h
     ABS_MT_POSITION_X = 0x35
 
     _cache_input = None
+    _cache_xinput = None
 
     class Input(object):
 
         def __init__(self, path):
+            query_xinput()
             self.path = path
 
         @property
@@ -80,7 +86,8 @@ else:
             long_bit = getconf("LONG_BIT")
             for i, word in enumerate(line.split(" ")):
                 word = int(word, 16)
-                subcapabilities = [bool(word & 1 << i) for i in range(long_bit)]
+                subcapabilities = [bool(word & 1 << i)
+                                   for i in range(long_bit)]
                 capabilities[:0] = subcapabilities
 
             return capabilities
@@ -89,10 +96,42 @@ else:
             capabilities = self.get_capabilities()
             return len(capabilities) > capability and capabilities[capability]
 
+        @property
+        def is_mouse(self):
+            return self.device in _cache_xinput
+
+    def getout(*args):
+        try:
+            return Popen(args, stdout=PIPE).communicate()[0]
+        except OSError:
+            return ''
+
     def getconf(var):
-        output = Popen(["getconf", var], stdout=PIPE).communicate()[0]
+        output = getout("getconf", var)
         return int(output)
 
+    def query_xinput():
+        global _cache_xinput
+        if _cache_xinput is None:
+            _cache_xinput = []
+            devids = getout('xinput', '--list', '--id-only')
+            for did in devids.splitlines():
+                devprops = getout('xinput', '--list-props', did)
+                evpath = None
+                for prop in devprops.splitlines():
+                    prop = prop.strip()
+                    if (prop.startswith(b'Device Enabled') and
+                            prop.endswith(b'0')):
+                        evpath = None
+                        break
+                    if prop.startswith(b'Device Node'):
+                        try:
+                            evpath = prop.split('"')[1]
+                        except Exception:
+                            evpath = None
+                if evpath:
+                    _cache_xinput.append(evpath)
+
     def get_inputs(path):
         global _cache_input
         if _cache_input is None:
@@ -120,7 +159,7 @@ else:
             self.provider = 'mtdev'
             self.match = None
             self.input_path = '/sys/class/input'
-            self.select_all = False
+            self.select_all = True if _is_rpi else False
             self.use_regex = False
             self.args = []
 
@@ -152,11 +191,23 @@ else:
 
             self.probe()
 
+        def should_use_mouse(self):
+            return not any(p for p in EventLoop.input_providers
+                           if isinstance(p, MouseMotionEventProvider))
+
         def probe(self):
+            global EventLoop
+            from kivy.base import EventLoop
+
             inputs = get_inputs(self.input_path)
+            Logger.debug('ProbeSysfs: using probesysfs!')
+
+            use_mouse = self.should_use_mouse()
+
             if not self.select_all:
                 inputs = [x for x in inputs if
-                          x.has_capability(ABS_MT_POSITION_X)]
+                          x.has_capability(ABS_MT_POSITION_X) and
+                          (use_mouse or not x.is_mouse)]
             for device in inputs:
                 Logger.debug('ProbeSysfs: found device: %s at %s' % (
                     device.name, device.device))
@@ -180,7 +231,7 @@ else:
                 provider = MotionEventFactory.get(self.provider)
                 if provider is None:
                     Logger.info('ProbeSysfs: unable to found provider %s' %
-                                     self.provider)
+                                self.provider)
                     Logger.info('ProbeSysfs: fallback on hidinput')
                     provider = MotionEventFactory.get('hidinput')
                 if provider is None:
@@ -188,10 +239,9 @@ else:
                                     ' to handle this device !')
                     continue
 
-                instance = provider(devicename, '%s,%s' % (device.device,
-                                                           ','.join(self.args)))
+                instance = provider(devicename, '%s,%s' % (
+                    device.device, ','.join(self.args)))
                 if instance:
-                    from kivy.base import EventLoop
                     EventLoop.add_input_provider(instance)
 
     MotionEventFactory.register('probesysfs', ProbeSysfsHardwareProbe)
