@@ -4,12 +4,12 @@ Asynchronous data loader
 
 This is the Asynchronous Loader. You can use it to load an image
 and use it, even if data are not yet available. You must specify a default
-loading image for using a such loader::
+loading image when using the loader::
 
-    from kivy import *
+    from kivy.loader import Loader
     image = Loader.image('mysprite.png')
 
-You can also load image from url::
+You can also load an image from a url::
 
     image = Loader.image('http://mysite.com/test.png')
 
@@ -22,14 +22,14 @@ Tweaking the asynchronous loader
 
 .. versionadded:: 1.6.0
 
-You can now tweak the loader to have a better user experience or more
-performance, depending of the images you're gonna to load. Take a look at the
+You can tweak the loader to provide a better user experience or more
+performance, depending of the images you are going to load. Take a look at the
 parameters:
 
-- :data:`Loader.num_workers` - define the number of threads to start for
-  loading images
-- :data:`Loader.max_upload_per_frame` - define the maximum image uploads in
-  GPU to do per frames.
+- :attr:`Loader.num_workers` - define the number of threads to start for
+  loading images.
+- :attr:`Loader.max_upload_per_frame` - define the maximum image uploads in
+  GPU to do per frame.
 
 '''
 
@@ -40,12 +40,14 @@ from kivy.logger import Logger
 from kivy.clock import Clock
 from kivy.cache import Cache
 from kivy.core.image import ImageLoader, Image
+from kivy.compat import PY2, string_types
 
 from collections import deque
 from time import sleep
 from os.path import join
 from os import write, close, unlink, environ
 import threading
+import mimetypes
 
 # Register a cache for loader
 Cache.register('kv.loader', limit=500, timeout=60)
@@ -55,31 +57,39 @@ class ProxyImage(Image):
     '''Image returned by the Loader.image() function.
 
     :Properties:
-        `loaded`: bool, default to False
-            It can be True if the image is already cached
+        `loaded`: bool, defaults to False
+            This value may be True if the image is already cached.
 
     :Events:
         `on_load`
-            Fired when the image is loaded and changed
+            Fired when the image is loaded or changed.
+        `on_error`
+            Fired when the image cannot be loaded.
+            `error`: Exception data that ocurred
     '''
 
+    __events__ = ('on_load', 'on_error')
+
     def __init__(self, arg, **kwargs):
-        kwargs.setdefault('loaded', False)
+        loaded = kwargs.pop('loaded', False)
         super(ProxyImage, self).__init__(arg, **kwargs)
-        self.loaded = kwargs.get('loaded')
-        self.register_event_type('on_load')
+        self.loaded = loaded
 
     def on_load(self):
         pass
 
+    def on_error(self, error):
+        pass
+
 
 class LoaderBase(object):
-    '''Common base for Loader and specific implementation.
-    By default, Loader will be the best available loader implementation.
+    '''Common base for the Loader and specific implementations.
+    By default, the Loader will be the best available loader implementation.
 
     The _update() function is called every 1 / 25.s or each frame if we have
     less than 25 FPS.
     '''
+    _trigger_update = None
 
     def __init__(self):
         self._loading_image = None
@@ -97,22 +107,21 @@ class LoaderBase(object):
         self._trigger_update = Clock.create_trigger(self._update)
 
     def __del__(self):
-        try:
-            Clock.unschedule(self._update)
-        except Exception:
-            pass
+        if self._trigger_update is not None:
+            self._trigger_update.cancel()
 
     def _set_num_workers(self, num):
         if num < 2:
             raise Exception('Must have at least 2 workers')
         self._num_workers = num
+
     def _get_num_workers(self):
         return self._num_workers
 
     num_workers = property(_get_num_workers, _set_num_workers)
-    '''Number of workers to use while loading. (used only if the loader
-    implementation support it.). This setting impact the loader only at the
-    beginning. Once the loader is started, the setting has no impact::
+    '''Number of workers to use while loading (used only if the loader
+    implementation supports it). This setting impacts the loader only on
+    initialization. Once the loader is started, the setting has no impact::
 
         from kivy.loader import Loader
         Loader.num_workers = 4
@@ -121,7 +130,7 @@ class LoaderBase(object):
     increase the number of workers, then all the images will be loaded faster,
     but the user will not been able to use the application while loading.
     Prior to 1.6.0, the default number was 20, and loading many full-hd images
-    was blocking completly the application.
+    was completly blocking the application.
 
     .. versionadded:: 1.6.0
     '''
@@ -130,23 +139,25 @@ class LoaderBase(object):
         if num is not None and num < 1:
             raise Exception('Must have at least 1 image processing per image')
         self._max_upload_per_frame = num
+
     def _get_max_upload_per_frame(self):
         return self._max_upload_per_frame
 
     max_upload_per_frame = property(_get_max_upload_per_frame,
-            _set_max_upload_per_frame)
-    '''Number of image to upload per frame. By default, we'll upload only 2
-    images in the GPU per frame. If you are uploading many tiny images, you can
-    easily increase this parameter to 10, or more.
-    If you are loading multiples Full-HD images, the upload time can be
-    consequent, and can stuck the application during the upload. If you want a
-    smooth experience, let the default.
+                                    _set_max_upload_per_frame)
+    '''The number of images to upload per frame. By default, we'll
+    upload only 2 images to the GPU per frame. If you are uploading many
+    small images, you can easily increase this parameter to 10 or more.
+    If you are loading multiple full HD images, the upload time may have
+    consequences and block the application. If you want a
+    smooth experience, use the default.
 
-    As matter of fact, a Full-HD RGB image will take ~6MB in memory, so it will
-    take times. If you have activated mipmap=True too, then the GPU must
-    calculate the mipmap of this big images too, in real time. Then it can be
-    smart to reduce the :data:`max_upload_per_frame` to 1 or 2. If you get ride
-    of that (or reduce it a lot), take a look at the DDS format.
+    As a matter of fact, a Full-HD RGB image will take ~6MB in memory,
+    so it may take time. If you have activated mipmap=True too, then the
+    GPU must calculate the mipmap of these big images too, in real time.
+    Then it may be best to reduce the :attr:`max_upload_per_frame` to 1
+    or 2. If you want to get rid of that (or reduce it a lot), take a
+    look at the DDS format.
 
     .. versionadded:: 1.6.0
     '''
@@ -158,7 +169,7 @@ class LoaderBase(object):
         return self._loading_image
 
     def _set_loading_image(self, image):
-        if isinstance(image, basestring):
+        if isinstance(image, string_types):
             self._loading_image = ImageLoader.load(filename=image)
         else:
             self._loading_image = image
@@ -181,7 +192,7 @@ class LoaderBase(object):
         return self._error_image
 
     def _set_error_image(self, image):
-        if isinstance(image, basestring):
+        if isinstance(image, string_types):
             self._error_image = ImageLoader.load(filename=image)
         else:
             self._error_image = image
@@ -191,13 +202,13 @@ class LoaderBase(object):
     You can change it by doing::
 
         Loader.error_image = 'error.png'
-        
+
     .. versionchanged:: 1.6.0
         Not readonly anymore.
     '''
 
     def start(self):
-        '''Start the loader thread/process'''
+        '''Start the loader thread/process.'''
         self._running = True
 
     def run(self, *largs):
@@ -205,11 +216,11 @@ class LoaderBase(object):
         pass
 
     def stop(self):
-        '''Stop the loader thread/process'''
+        '''Stop the loader thread/process.'''
         self._running = False
 
     def pause(self):
-        '''Pause the loader, can be useful during interactions
+        '''Pause the loader, can be useful during interactions.
 
         .. versionadded:: 1.6.0
         '''
@@ -234,9 +245,11 @@ class LoaderBase(object):
     def _load(self, kwargs):
         '''(internal) Loading function, called by the thread.
         Will call _load_local() if the file is local,
-        or _load_urllib() if the file is on Internet'''
+        or _load_urllib() if the file is on Internet.
+        '''
 
-        while len(self._q_done) >= self.max_upload_per_frame * self._num_workers:
+        while len(self._q_done) >= (
+                self.max_upload_per_frame * self._num_workers):
             sleep(0.1)
 
         self._wait_for_resume()
@@ -247,7 +260,7 @@ class LoaderBase(object):
         try:
             proto = filename.split(':', 1)[0]
         except:
-            #if blank filename then return
+            # if blank filename then return
             return
         if load_callback is not None:
             data = load_callback(filename)
@@ -270,13 +283,22 @@ class LoaderBase(object):
 
     def _load_urllib(self, filename, kwargs):
         '''(internal) Loading a network file. First download it, save it to a
-        temporary file, and pass it to _load_local()'''
-        import urllib2
+        temporary file, and pass it to _load_local().'''
+        if PY2:
+            import urllib2 as urllib_request
+
+            def gettype(info):
+                return info.gettype()
+        else:
+            import urllib.request as urllib_request
+
+            def gettype(info):
+                return info.get_content_type()
         proto = filename.split(':', 1)[0]
         if proto == 'smb':
             try:
                 # note: it's important to load SMBHandler every time
-                # otherwise the data is occasionaly not loaded
+                # otherwise the data is occasionally not loaded
                 from smb.SMBHandler import SMBHandler
             except ImportError:
                 Logger.warning(
@@ -286,16 +308,32 @@ class LoaderBase(object):
         data = fd = _out_osfd = None
         try:
             _out_filename = ''
-            suffix = '.%s' % (filename.split('.')[-1])
-            _out_osfd, _out_filename = tempfile.mkstemp(
-                    prefix='kivyloader', suffix=suffix)
 
             if proto == 'smb':
                 # read from samba shares
-                fd = urllib2.build_opener(SMBHandler).open(filename)
+                fd = urllib_request.build_opener(SMBHandler).open(filename)
             else:
                 # read from internet
-                fd = urllib2.urlopen(filename)
+                fd = urllib_request.urlopen(filename)
+
+            if '#.' in filename:
+                # allow extension override from URL fragment
+                suffix = '.' + filename.split('#.')[-1]
+            else:
+                ctype = gettype(fd.info())
+                suffix = mimetypes.guess_extension(ctype)
+                if not suffix:
+                    # strip query string and split on path
+                    parts = filename.split('?')[0].split('/')[1:]
+                    while len(parts) > 1 and not parts[0]:
+                        # strip out blanks from '//'
+                        parts = parts[1:]
+                    if len(parts) > 1 and '.' in parts[-1]:
+                        # we don't want '.com', '.net', etc. as the extension
+                        suffix = '.' + parts[-1].split('.')[-1]
+            _out_osfd, _out_filename = tempfile.mkstemp(
+                prefix='kivyloader', suffix=suffix)
+
             idata = fd.read()
             fd.close()
             fd = None
@@ -311,13 +349,24 @@ class LoaderBase(object):
             # FIXME create a clean API for that
             for imdata in data._data:
                 imdata.source = filename
-        except Exception:
-            Logger.exception('Failed to load image <%s>' % filename)
+        except Exception as ex:
+            Logger.exception('Loader: Failed to load image <%s>' % filename)
             # close file when remote file not found or download error
             try:
-                close(_out_osfd)
+                if _out_osfd:
+                    close(_out_osfd)
             except OSError:
                 pass
+
+            # update client
+            for c_filename, client in self._client[:]:
+                if filename != c_filename:
+                    continue
+                # got one client to update
+                client.image = self.error_image
+                client.dispatch('on_error', error=ex)
+                self._client.remove((c_filename, client))
+
             return self.error_image
         finally:
             if fd:
@@ -330,7 +379,7 @@ class LoaderBase(object):
         return data
 
     def _update(self, *largs):
-        '''(internal) Check if a data is loaded, and pass to the client'''
+        '''(internal) Check if a data is loaded, and pass to the client.'''
         # want to start it ?
         if self._start_wanted:
             if not self._running:
@@ -342,7 +391,7 @@ class LoaderBase(object):
             self._trigger_update()
             return
 
-        for x in xrange(self.max_upload_per_frame):
+        for x in range(self.max_upload_per_frame):
             try:
                 filename, data = self._q_done.pop()
             except IndexError:
@@ -365,28 +414,39 @@ class LoaderBase(object):
 
         self._trigger_update()
 
-    def image(self, filename, load_callback=None, post_callback=None, **kwargs):
-        '''Load a image using loader. A Proxy image is returned with a loading
-        image.
+    def image(self, filename, load_callback=None, post_callback=None,
+              **kwargs):
+        '''Load a image using the Loader. A ProxyImage is returned with a
+        loading image. You can use it as follows::
 
-      ::
-            img = Loader.image(filename)
-            # img will be a ProxyImage.
-            # You'll use it the same as an Image class.
-            # Later, when the image is really loaded,
-            # the loader will change the img.image property
-            # to the new loaded image
+            from kivy.app import App
+            from kivy.uix.image import Image
+            from kivy.loader import Loader
 
+            class TestApp(App):
+                def _image_loaded(self, proxyImage):
+                    if proxyImage.image.texture:
+                        self.image.texture = proxyImage.image.texture
+
+                def build(self):
+                    proxyImage = Loader.image("myPic.jpg")
+                    proxyImage.bind(on_load=self._image_loaded)
+                    self.image = Image()
+                    return self.image
+
+            TestApp().run()
+
+        In order to cancel all background loading, call *Loader.stop()*.
         '''
         data = Cache.get('kv.loader', filename)
         if data not in (None, False):
             # found image, if data is not here, need to reload.
             return ProxyImage(data,
-                    loading_image=self.loading_image,
-                    loaded=True, **kwargs)
+                              loading_image=self.loading_image,
+                              loaded=True, **kwargs)
 
         client = ProxyImage(self.loading_image,
-                    loading_image=self.loading_image, **kwargs)
+                            loading_image=self.loading_image, **kwargs)
         self._client.append((filename, client))
 
         if data is None:
@@ -410,6 +470,7 @@ class LoaderBase(object):
 # Loader implementation
 #
 
+
 if 'KIVY_DOC' in environ:
 
     Loader = None
@@ -420,10 +481,9 @@ else:
     # Try to use pygame as our first choice for loader
     #
 
-    from Queue import Queue
+    from kivy.compat import queue
     from threading import Thread
 
-
     class _Worker(Thread):
         '''Thread executing tasks from a given tasks queue
         '''
@@ -437,18 +497,19 @@ else:
         def run(self):
             while self.pool.running:
                 func, args, kargs = self.tasks.get()
-                try: func(*args, **kargs)
-                except Exception, e: print e
+                try:
+                    func(*args, **kargs)
+                except Exception as e:
+                    print(e)
                 self.tasks.task_done()
 
-
     class _ThreadPool(object):
         '''Pool of threads consuming tasks from a queue
         '''
         def __init__(self, num_threads):
             super(_ThreadPool, self).__init__()
             self.running = True
-            self.tasks = Queue()
+            self.tasks = queue.Queue()
             for _ in range(num_threads):
                 _Worker(self, self.tasks)
 
@@ -461,7 +522,6 @@ else:
             self.running = False
             self.tasks.join()
 
-
     class LoaderThreadPool(LoaderBase):
         def __init__(self):
             super(LoaderThreadPool, self).__init__()
@@ -485,8 +545,6 @@ else:
                     return
                 self.pool.add_task(self._load, parameters)
 
-
     Loader = LoaderThreadPool()
     Logger.info('Loader: using a thread pool of {} workers'.format(
         Loader.num_workers))
-
