@@ -2,23 +2,24 @@
 Logger object
 =============
 
-Differents level are available : trace, debug, info, warning, error, critical.
+Differents logging levels are available : trace, debug, info, warning, error
+and critical.
 
 Examples of usage::
 
     from kivy.logger import Logger
 
-    Logger.info('title: This is a info')
-    Logger.debug('title: This is a debug')
+    Logger.info('title: This is a info message.')
+    Logger.debug('title: This is a debug message.')
 
     try:
         raise Exception('bleh')
-    except Exception, e:
-        Logger.exception(e)
+    except Exception:
+        Logger.exception('Something happened!')
 
-The message passed to the logger is splited to the first :. The left part is
-used as a title, and the right part is used as a message. This way, you can
-"categorize" your message easily::
+The message passed to the logger is split into two parts, separated by a colon
+(:). The first part is used as a title, and the second part is used as the
+message. This way, you can "categorize" your message easily. ::
 
     Logger.info('Application: This is a test')
 
@@ -29,26 +30,27 @@ used as a title, and the right part is used as a message. This way, you can
 Logger configuration
 --------------------
 
-Logger can be controled in the Kivy configuration file::
+The Logger can be controlled via the Kivy configuration file::
 
     [kivy]
     log_level = info
     log_enable = 1
     log_dir = logs
     log_name = kivy_%y-%m-%d_%_.txt
+    log_maxfile = 100
 
-More information about the allowed values is described in :mod:`kivy.config`
-module.
+More information about the allowed values are described in the
+:mod:`kivy.config` module.
 
 Logger history
 --------------
 
-Even if the logger is not enabled, you can still have the history of latest 100
+Even if the logger is not enabled, you still have access to the last 100
 messages::
 
     from kivy.logger import LoggerHistory
 
-    print LoggerHistory.history
+    print(LoggerHistory.history)
 
 '''
 
@@ -56,6 +58,7 @@ import logging
 import os
 import sys
 import kivy
+from kivy.compat import PY2
 from random import randint
 from functools import partial
 
@@ -63,9 +66,9 @@ __all__ = ('Logger', 'LOG_LEVELS', 'COLORS', 'LoggerHistory')
 
 Logger = None
 
-BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
+BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = list(range(8))
 
-#These are the sequences need to get colored ouput
+# These are the sequences need to get colored ouput
 RESET_SEQ = "\033[0m"
 COLOR_SEQ = "\033[1;%dm"
 BOLD_SEQ = "\033[1m"
@@ -81,6 +84,7 @@ def formatter_message(message, use_color=True):
         message = message.replace("$RESET", "").replace("$BOLD", "")
     return message
 
+
 COLORS = {
     'TRACE': MAGENTA,
     'WARNING': YELLOW,
@@ -105,52 +109,58 @@ class FileHandler(logging.Handler):
     fd = None
 
     def purge_logs(self, directory):
-        '''Purge log is called randomly, to prevent log directory to be filled
-        by lot and lot of log files.
-        You've a chance of 1 on 20 to fire a purge log.
+        '''Purge log is called randomly to prevent the log directory from being
+        filled by lots and lots of log files.
+        You've a chance of 1 in 20 that purge log will be fired.
         '''
         if randint(0, 20) != 0:
             return
 
-        # Use config ?
-        maxfiles = 100
+        from kivy.config import Config
+        maxfiles = Config.getint('kivy', 'log_maxfiles')
 
-        print 'Purge log fired. Analysing...'
+        if maxfiles < 0:
+            return
+
+        print('Purge log fired. Analysing...')
         join = os.path.join
         unlink = os.unlink
 
         # search all log files
-        l = map(lambda x: join(directory, x), os.listdir(directory))
-        if len(l) > maxfiles:
+        lst = [join(directory, x) for x in os.listdir(directory)]
+        if len(lst) > maxfiles:
             # get creation time on every files
-            l = zip(l, map(os.path.getctime, l))
+            lst = [{'fn': x, 'ctime': os.path.getctime(x)} for x in lst]
 
             # sort by date
-            l.sort(cmp=lambda x, y: cmp(x[1], y[1]))
+            lst = sorted(lst, key=lambda x: x['ctime'])
 
             # get the oldest (keep last maxfiles)
-            l = l[:-maxfiles]
-            print 'Purge %d log files' % len(l)
+            lst = lst[:-maxfiles] if maxfiles else lst
+            print('Purge %d log files' % len(lst))
 
-            # now, unlink every files in the list
-            for filename in l:
-                unlink(filename[0])
+            # now, unlink every file in the list
+            for filename in lst:
+                try:
+                    unlink(filename['fn'])
+                except PermissionError as e:
+                    print('Skipped file {0}, {1}'.format(filename['fn'], e))
 
-        print 'Purge finished !'
+        print('Purge finished!')
 
-    def _configure(self):
+    def _configure(self, *largs, **kwargs):
         from time import strftime
         from kivy.config import Config
         log_dir = Config.get('kivy', 'log_dir')
         log_name = Config.get('kivy', 'log_name')
 
         _dir = kivy.kivy_home_dir
-        if len(log_dir) and log_dir[0] == '/':
+        if log_dir and os.path.isabs(log_dir):
             _dir = log_dir
         else:
             _dir = os.path.join(_dir, log_dir)
-            if not os.path.exists(_dir):
-                os.mkdir(_dir)
+        if not os.path.exists(_dir):
+            os.makedirs(_dir)
 
         self.purge_logs(_dir)
 
@@ -165,7 +175,11 @@ class FileHandler(logging.Handler):
             if n > 10000:  # prevent maybe flooding ?
                 raise Exception('Too many logfile, remove them')
 
+        if FileHandler.filename == filename and FileHandler.fd is not None:
+            return
         FileHandler.filename = filename
+        if FileHandler.fd is not None:
+            FileHandler.fd.close()
         FileHandler.fd = open(filename, 'w')
 
         Logger.info('Logger: Record log in %s' % filename)
@@ -174,28 +188,51 @@ class FileHandler(logging.Handler):
         if FileHandler.fd in (None, False):
             return
 
-        FileHandler.fd.write('[%-18s] ' % record.levelname)
-        try:
-            FileHandler.fd.write(record.msg)
-        except UnicodeEncodeError:
-            FileHandler.fd.write(record.msg.encode('utf8'))
-        FileHandler.fd.write('\n')
-        FileHandler.fd.flush()
+        msg = self.format(record)
+        stream = FileHandler.fd
+        fs = "%s\n"
+        stream.write('[%-7s] ' % record.levelname)
+        if PY2:
+            try:
+                if (isinstance(msg, unicode) and
+                        getattr(stream, 'encoding', None)):
+                    ufs = u'%s\n'
+                    try:
+                        stream.write(ufs % msg)
+                    except UnicodeEncodeError:
+                        stream.write((ufs % msg).encode(stream.encoding))
+                else:
+                    stream.write(fs % msg)
+            except UnicodeError:
+                stream.write(fs % msg.encode("UTF-8"))
+        else:
+            stream.write(fs % msg)
+        stream.flush()
 
     def emit(self, message):
-        if not Logger.logfile_activated:
+        # during the startup, store the message in the history
+        if Logger.logfile_activated is None:
             FileHandler.history += [message]
             return
 
+        # startup done, if the logfile is not activated, avoid history.
+        if Logger.logfile_activated is False:
+            FileHandler.history = []
+            return
+
         if FileHandler.fd is None:
             try:
                 self._configure()
+                from kivy.config import Config
+                Config.add_callback(self._configure, 'kivy', 'log_dir')
+                Config.add_callback(self._configure, 'kivy', 'log_name')
             except Exception:
                 # deactivate filehandler...
                 FileHandler.fd = False
                 Logger.exception('Error while activating FileHandler logger')
                 return
-            for _message in FileHandler.history:
+            while FileHandler.history:
+                _message = FileHandler.history.pop()
                 self._write_message(_message)
 
         self._write_message(message)
@@ -216,11 +253,6 @@ class ColoredFormatter(logging.Formatter):
         self.use_color = use_color
 
     def format(self, record):
-        # XXX Hack to not show the fucking traceback for Numeric handler
-        # Lot of people are complaining with that. Now we did.
-        if 'Unable to load registered array format handler' in record.msg:
-            if record.args and record.args[0] == 'numeric':
-                return
         try:
             msg = record.msg.split(':', 1)
             if len(msg) == 2:
@@ -258,6 +290,7 @@ class LogFile(object):
         self.buffer = ''
         self.func = func
         self.channel = channel
+        self.errors = ''
 
     def write(self, s):
         s = self.buffer + s
@@ -272,17 +305,24 @@ class LogFile(object):
     def flush(self):
         return
 
+    def isatty(self):
+        return False
+
 
 def logger_config_update(section, key, value):
     if LOG_LEVELS.get(value) is None:
         raise AttributeError('Loglevel {0!r} doesn\'t exists'.format(value))
     Logger.setLevel(level=LOG_LEVELS.get(value))
 
+
 #: Kivy default logger instance
 Logger = logging.getLogger('kivy')
-Logger.logfile_activated = False
+Logger.logfile_activated = None
 Logger.trace = partial(Logger.log, logging.TRACE)
 
+# set the Kivy logger as the default
+logging.root = Logger
+
 # add default kivy logger
 Logger.addHandler(LoggerHistory())
 if 'KIVY_NO_FILELOG' not in os.environ:
@@ -293,11 +333,28 @@ if 'KIVY_NO_CONSOLELOG' not in os.environ:
     if hasattr(sys, '_kivy_logging_handler'):
         Logger.addHandler(getattr(sys, '_kivy_logging_handler'))
     else:
-        use_color = os.name != 'nt'
-        if os.environ.get('KIVY_BUILD') in ('android', 'ios'):
-            use_color = False
-        color_fmt = formatter_message(
-            '[%(levelname)-18s] %(message)s', use_color)
+        use_color = (
+            os.name != 'nt' and
+            os.environ.get('KIVY_BUILD') not in ('android', 'ios') and
+            os.environ.get('TERM') in (
+                'rxvt',
+                'rxvt-256color',
+                'rxvt-unicode',
+                'rxvt-unicode-256color',
+                'xterm',
+                'xterm-256color',
+            )
+        )
+        if not use_color:
+            # No additional control characters will be inserted inside the
+            # levelname field, 7 chars will fit "WARNING"
+            color_fmt = formatter_message(
+                '[%(levelname)-7s] %(message)s', use_color)
+        else:
+            # levelname field width need to take into account the length of the
+            # color control codes (7+4 chars for bold+color, and reset)
+            color_fmt = formatter_message(
+                '[%(levelname)-18s] %(message)s', use_color)
         formatter = ColoredFormatter(color_fmt, use_color=use_color)
         console = ConsoleHandler()
         console.setFormatter(formatter)
