@@ -2,20 +2,26 @@
 Support
 =======
 
-Activate other framework/toolkit inside our event loop
+Activate other frameworks/toolkits inside the kivy event loop.
+
 '''
 
 __all__ = ('install_gobject_iteration', 'install_twisted_reactor',
-    'install_android')
+           'uninstall_twisted_reactor', 'install_android')
 
 
 def install_gobject_iteration():
     '''Import and install gobject context iteration inside our event loop.
-    This is used as soon as gobject is used (like gstreamer)
+    This is used as soon as gobject is used (like gstreamer).
     '''
 
     from kivy.clock import Clock
-    import gobject
+
+    try:
+        from gi.repository import GObject as gobject
+    except ImportError:
+        import gobject
+
     if hasattr(gobject, '_gobject_already_installed'):
         # already installed, don't do it twice.
         return
@@ -29,18 +35,20 @@ def install_gobject_iteration():
 
     # schedule the iteration each frame
     def _gobject_iteration(*largs):
-        # XXX we need to loop over context here, otherwise, we might have a lag.
+        # XXX we need to loop over context here, otherwise, we might have a lag
         loop = 0
         while context.pending() and loop < 10:
             context.iteration(False)
             loop += 1
     Clock.schedule_interval(_gobject_iteration, 0)
 
+
 # -----------------------------------------------------------------------------
 # Android support
 # -----------------------------------------------------------------------------
 
 g_android_redraw_count = 0
+_redraw_event = None
 
 
 def _android_ask_redraw(*largs):
@@ -55,15 +63,15 @@ def _android_ask_redraw(*largs):
 
 
 def install_android():
-    '''Install hooks for android platform.
+    '''Install hooks for the android platform.
 
-    * Automaticly sleep when the device is paused
-    * Auto kill the application is the return key is hitted
+    * Automatically sleep when the device is paused.
+    * Automatically kill the application when the return key is pressed.
     '''
     try:
         import android
     except ImportError:
-        print 'Android lib is missing, cannot install android hooks'
+        print('Android lib is missing, cannot install android hooks')
         return
 
     from kivy.clock import Clock
@@ -77,10 +85,10 @@ def install_android():
     android.map_key(android.KEYCODE_MENU, pygame.K_MENU)
     android.map_key(android.KEYCODE_BACK, pygame.K_ESCAPE)
 
-    # Check if android must be paused or not
-    # If pause is asked, just leave the app.
+    # Check if android should be paused or not.
+    # If pause is requested, just leave the app.
     def android_check_pause(*largs):
-        # do nothing until android ask for it.
+        # do nothing until android asks for it.
         if not android.check_pause():
             return
 
@@ -88,10 +96,10 @@ def install_android():
         from kivy.base import stopTouchApp
         from kivy.logger import Logger
         from kivy.core.window import Window
-        global g_android_redraw_count
+        global g_android_redraw_count, _redraw_event
 
         # try to get the current running application
-        Logger.info('Android: Must to in sleep mode, check the app')
+        Logger.info('Android: Must go into sleep mode, check the app')
         app = App.get_running_app()
 
         # no running application, stop our loop.
@@ -110,19 +118,23 @@ def install_android():
             # is it a stop or resume ?
             if android.check_stop():
                 # app must stop
-                Logger.info('Android: Android want to close our app.')
+                Logger.info('Android: Android wants to close our app.')
                 stopTouchApp()
             else:
                 # app resuming now !
-                Logger.info('Android: Android resumed, resume the app')
+                Logger.info('Android: Android has resumed, resume the app.')
                 app.dispatch('on_resume')
                 Window.canvas.ask_update()
-                g_android_redraw_count = 25 # 5 frames/seconds, during 5 seconds
-                Clock.unschedule(_android_ask_redraw)
-                Clock.schedule_interval(_android_ask_redraw, 1 / 5)
+                g_android_redraw_count = 25  # 5 frames/seconds for 5 seconds
+                if _redraw_event is None:
+                    _redraw_event = Clock.schedule_interval(
+                        _android_ask_redraw, 1 / 5)
+                else:
+                    _redraw_event.cancel()
+                    _redraw_event()
                 Logger.info('Android: App resume completed.')
 
-        # app don't support pause mode, just stop it.
+        # app doesn't support pause mode, just stop it.
         else:
             Logger.info('Android: App doesn\'t support pause mode, stop.')
             stopTouchApp()
@@ -130,20 +142,36 @@ def install_android():
     Clock.schedule_interval(android_check_pause, 0)
 
 
+_twisted_reactor_stopper = None
+_twisted_reactor_work = None
+
+
 def install_twisted_reactor(**kwargs):
     '''Installs a threaded twisted reactor, which will schedule one
     reactor iteration before the next frame only when twisted needs
     to do some work.
 
-    any arguments or keyword arguments passed to this function will be
-    passed on the the threadedselect reactors interleave function, these
-    are the arguments one would usually pass to twisted's reactor.startRunning
+    Any arguments or keyword arguments passed to this function will be
+    passed on the the threadedselect reactors interleave function. These
+    are the arguments one would usually pass to twisted's reactor.startRunning.
 
     Unlike the default twisted reactor, the installed reactor will not handle
-    any signals unnless you set the 'installSignalHandlers' keyword argument
-    to 1 explicitly.  This is done to allow kivy to handle teh signals as
-    usual, unless you specifically want the twisted reactor to handle the
-    signals (e.g. SIGINT).'''
+    any signals unless you set the 'installSignalHandlers' keyword argument
+    to 1 explicitly. This is done to allow kivy to handle the signals as
+    usual unless you specifically want the twisted reactor to handle the
+    signals (e.g. SIGINT).
+
+    .. note::
+        Twisted is not included in iOS build by default. To use it on iOS,
+        put the twisted distribution (and zope.interface dependency) in your
+        application directory.
+    '''
+    import sys
+    if sys.version_info[0] > 2:
+        raise NotImplementedError(
+            "_threadselect interface of twisted is not available on Python 3."
+        )
+
     import twisted
 
     # prevent installing more than once
@@ -151,7 +179,7 @@ def install_twisted_reactor(**kwargs):
         return
     twisted._kivy_twisted_reactor_installed = True
 
-    # dont let twisted handle signals, unless specifically requested
+    # don't let twisted handle signals, unless specifically requested
     kwargs.setdefault('installSignalHandlers', 0)
 
     # install threaded-select reactor, to use with own event loop
@@ -160,6 +188,8 @@ def install_twisted_reactor(**kwargs):
 
     # now we can import twisted reactor as usual
     from twisted.internet import reactor
+    from twisted.internet.error import ReactorNotRunning
+
     from collections import deque
     from kivy.base import EventLoop
     from kivy.logger import Logger
@@ -168,34 +198,75 @@ def install_twisted_reactor(**kwargs):
     # will hold callbacks to twisted callbacks
     q = deque()
 
-    # twisted will call the wake function when it needsto do work
+    # twisted will call the wake function when it needs to do work
     def reactor_wake(twisted_loop_next):
+        '''Wakeup the twisted reactor to start processing the task queue
+        '''
+
         Logger.trace("Support: twisted wakeup call to schedule task")
         q.append(twisted_loop_next)
 
     # called every frame, to process the reactors work in main thread
     def reactor_work(*args):
+        '''Process the twisted reactor task queue
+        '''
         Logger.trace("Support: processing twisted task queue")
         while len(q):
             q.popleft()()
+    global _twisted_reactor_work
+    _twisted_reactor_work = reactor_work
 
     # start the reactor, by telling twisted how to wake, and process
     def reactor_start(*args):
+        '''Start the twisted reactor main loop
+        '''
         Logger.info("Support: Starting twisted reactor")
         reactor.interleave(reactor_wake, **kwargs)
         Clock.schedule_interval(reactor_work, 0)
 
     # make sure twisted reactor is shutdown if eventloop exists
     def reactor_stop(*args):
-        '''will shutdown the twisted reactor main loop
+        '''Shutdown the twisted reactor main loop
         '''
         if reactor.threadpool:
-            Logger.info("Support: Stooping twisted threads")
+            Logger.info("Support: Stopping twisted threads")
             reactor.threadpool.stop()
         Logger.info("Support: Shutting down twisted reactor")
         reactor._mainLoopShutdown()
+        try:
+            reactor.stop()
+        except ReactorNotRunning:
+            pass
 
-    # start and stop teh reactor along with kivy EventLoop
-    EventLoop.bind(on_start=reactor_start)
+        import sys
+        sys.modules.pop('twisted.internet.reactor', None)
+
+    global _twisted_reactor_stopper
+    _twisted_reactor_stopper = reactor_stop
+
+    # start and stop the reactor along with kivy EventLoop
+    Clock.schedule_once(reactor_start, 0)
     EventLoop.bind(on_stop=reactor_stop)
 
+
+def uninstall_twisted_reactor():
+    '''Uninstalls the Kivy's threaded Twisted Reactor. No more Twisted
+    tasks will run after this got called. Use this to clean the
+    `twisted.internet.reactor` .
+
+    .. versionadded:: 1.9.0
+    '''
+
+    import twisted
+
+    # prevent uninstalling more than once
+    if not hasattr(twisted, '_kivy_twisted_reactor_installed'):
+        return
+
+    from kivy.base import EventLoop
+
+    global _twisted_reactor_stopper
+    _twisted_reactor_stopper()
+    EventLoop.unbind(on_stop=_twisted_reactor_stopper)
+
+    del twisted._kivy_twisted_reactor_installed
