@@ -8,10 +8,15 @@ reStructuredText renderer
 easy-to-read, what-you-see-is-what-you-get plaintext markup syntax and parser
 system.
 
+.. note::
+
+    This widget requires the ``docutils`` package to run. Install it with
+    ``pip`` or include it as one of your deployment requirements.
+
 .. warning::
 
-    This widget is highly experimental. The whole styling and implementation are
-    not stable until this warning has been removed.
+    This widget is highly experimental. The styling and implementation should
+    not be considered stable until this warning has been removed.
 
 Usage with Text
 ---------------
@@ -27,7 +32,7 @@ Usage with Text
     This is an **emphased text**, some ``interpreted text``.
     And this is a reference to top_::
 
-        $ print "Hello world"
+        $ print("Hello world")
 
     """
     document = RstDocument(text=text)
@@ -39,29 +44,31 @@ The rendering will output:
 Usage with Source
 -----------------
 
-You can also render a rst file by using :data:`RstDocument.source`::
+You can also render a rst file using the :attr:`~RstDocument.source` property::
 
     document = RstDocument(source='index.rst')
 
-You can reference other documents with the role ``:doc:``. For example, in the
+You can reference other documents using the role ``:doc:``. For example, in the
 document ``index.rst`` you can write::
 
     Go to my next document: :doc:`moreinfo.rst`
 
-It will generate a link that, when clicked, the document ``moreinfo.rst``
-will be loaded.
+It will generate a link that, when clicked, opens the ``moreinfo.rst``
+document.
 
 '''
 
 __all__ = ('RstDocument', )
 
 import os
-from os.path import dirname, join, exists
+from os.path import dirname, join, exists, abspath
 from kivy.clock import Clock
+from kivy.compat import PY2
 from kivy.properties import ObjectProperty, NumericProperty, \
-        DictProperty, ListProperty, StringProperty, \
-        BooleanProperty
+    DictProperty, ListProperty, StringProperty, \
+    BooleanProperty, OptionProperty, AliasProperty
 from kivy.lang import Builder
+from kivy.utils import get_hex_from_color, get_color_from_hex
 from kivy.uix.widget import Widget
 from kivy.uix.scrollview import ScrollView
 from kivy.uix.gridlayout import GridLayout
@@ -76,7 +83,7 @@ from docutils.parsers.rst import roles
 from docutils import nodes, frontend, utils
 from docutils.parsers.rst import Directive, directives
 from docutils.parsers.rst.roles import set_classes
-       
+
 
 #
 # Handle some additional roles
@@ -105,7 +112,7 @@ if 'KIVY_DOC' not in os.environ:
     generic_docroles = {
         'doc': role_doc}
 
-    for rolename, nodeclass in generic_docroles.iteritems():
+    for rolename, nodeclass in generic_docroles.items():
         generic = roles.GenericRole(rolename, nodeclass)
         role = roles.CustomRole(rolename, generic, {'classes': [rolename]})
         roles.register_local_role(rolename, role)
@@ -121,9 +128,9 @@ Builder.load_string('''
     content: content
     scatter: scatter
     do_scroll_x: False
-    canvas:
+    canvas.before:
         Color:
-            rgb: parse_color(root.colors['background'])
+            rgba: parse_color(root.colors['background'])
         Rectangle:
             pos: self.pos
             size: self.size
@@ -148,7 +155,9 @@ Builder.load_string('''
 <RstTitle>:
     markup: True
     valign: 'top'
-    font_size: sp(31 - self.section * 2)
+    font_size:
+        sp(self.document.base_font_size - self.section * (
+        self.document.base_font_size / 31.0 * 2))
     size_hint_y: None
     height: self.texture_size[1] + dp(20)
     text_size: self.width, None
@@ -156,7 +165,7 @@ Builder.load_string('''
 
     canvas:
         Color:
-            rgba: parse_color('204a9699')
+            rgba: parse_color(self.document.underline_color)
         Rectangle:
             pos: self.x, self.y + 5
             size: self.width, 1
@@ -168,6 +177,7 @@ Builder.load_string('''
     size_hint_y: None
     height: self.texture_size[1] + self.my
     text_size: self.width - self.mx, None
+    font_size: sp(self.document.base_font_size / 2.0)
 
 <RstTerm>:
     size_hint: None, None
@@ -180,6 +190,7 @@ Builder.load_string('''
         valign: 'top'
         size_hint: None, None
         size: self.texture_size[0] + dp(10), self.texture_size[1] + dp(10)
+        font_size: sp(root.document.base_font_size / 2.0)
 
 <RstBlockQuote>:
     cols: 2
@@ -216,7 +227,7 @@ Builder.load_string('''
         markup: True
         valign: 'top'
         text_size: self.width - 20, None
-        font_name: 'data/fonts/DroidSansMono.ttf'
+        font_name: 'data/fonts/RobotoMono-Regular.ttf'
         color: (0, 0, 0, 1)
 
 <RstList>:
@@ -288,11 +299,13 @@ Builder.load_string('''
     cols: 1
     size_hint_y: None
     height: self.minimum_height
+    font_size: sp(self.document.base_font_size / 2.0)
 
 <RstDefinition>:
     cols: 2
     size_hint_y: None
     height: self.minimum_height
+    font_size: sp(self.document.base_font_size / 2.0)
 
 <RstFieldList>:
     cols: 2
@@ -302,11 +315,12 @@ Builder.load_string('''
 <RstFieldName>:
     markup: True
     valign: 'top'
-    size_hint: None, 1
+    size_hint: 0.2, 1
     color: (0, 0, 0, 1)
     bold: True
-    text_size: self.width - 10, self.height - 10
+    text_size: self.width-10, self.height - 10
     valign: 'top'
+    font_size: sp(self.document.base_font_size / 2.0)
 
 <RstFieldBody>:
     cols: 1
@@ -353,6 +367,7 @@ Builder.load_string('''
     size_hint_x: None
     width: self.texture_size[0] + dp(10)
     text_size: None, self.height - dp(10)
+    font_size: sp(self.document.base_font_size / 2.0)
 
 <RstEmptySpace>:
     size_hint: 0.01, 0.01
@@ -360,6 +375,7 @@ Builder.load_string('''
 <RstDefinitionSpace>:
     size_hint: None, 0.1
     width: 50
+    font_size: sp(self.document.base_font_size / 2.0)
 
 <RstVideoPlayer>:
     options: {'allow_stretch': True}
@@ -385,37 +401,83 @@ class RstDocument(ScrollView):
     source = StringProperty(None)
     '''Filename of the RST document.
 
-    :data:`source` is a :class:`~kivy.properties.StringProperty`, default to
-    None.
+    :attr:`source` is a :class:`~kivy.properties.StringProperty` and
+    defaults to None.
+    '''
+
+    source_encoding = StringProperty('utf-8')
+    '''Encoding to be used for the :attr:`source` file.
+
+    :attr:`source_encoding` is a :class:`~kivy.properties.StringProperty` and
+    defaults to `utf-8`.
+
+    .. Note::
+        It is your responsibility to ensure that the value provided is a
+        valid codec supported by python.
+    '''
+
+    source_error = OptionProperty('strict',
+                                  options=('strict', 'ignore', 'replace',
+                                           'xmlcharrefreplace',
+                                           'backslashreplac'))
+    '''Error handling to be used while encoding the :attr:`source` file.
+
+    :attr:`source_error` is an :class:`~kivy.properties.OptionProperty` and
+    defaults to `strict`. Can be one of 'strict', 'ignore', 'replace',
+    'xmlcharrefreplace' or 'backslashreplac'.
     '''
 
     text = StringProperty(None)
     '''RST markup text of the document.
 
-    :data:`text` is a :class:`~kivy.properties.StringProperty`, default to None.
+    :attr:`text` is a :class:`~kivy.properties.StringProperty` and defaults to
+    None.
     '''
 
     document_root = StringProperty(None)
-    '''Root path where :doc: will search any rst document. If no path is
-    given, then it will use the directory of the first loaded source.
+    '''Root path where :doc: will search for rst documents. If no path is
+    given, it will use the directory of the first loaded source file.
 
-    :data:`document_root` is a :class:`~kivy.properties.StringProperty`, default
-    to None.
+    :attr:`document_root` is a :class:`~kivy.properties.StringProperty` and
+    defaults to None.
+    '''
+
+    base_font_size = NumericProperty(31)
+    '''Font size for the biggest title, 31 by default. All other font sizes are
+    derived from this.
+
+    .. versionadded:: 1.8.0
     '''
 
     show_errors = BooleanProperty(False)
-    '''Indicate if RST parsers errors must be shown on the screen or not.
+    '''Indicate whether RST parsers errors should be shown on the screen
+    or not.
 
-    :data:`show_errors` is a :class:`~kivy.properties.BooleanProperty`, default
-    to False
+    :attr:`show_errors` is a :class:`~kivy.properties.BooleanProperty` and
+    defaults to False.
+    '''
+
+    def _get_bgc(self):
+        return get_color_from_hex(self.colors.background)
+
+    def _set_bgc(self, value):
+        self.colors.background = get_hex_from_color(value)[1:]
+
+    background_color = AliasProperty(_get_bgc, _set_bgc, bind=('colors',))
+    '''Specifies the background_color to be used for the RstDocument.
+
+    .. versionadded:: 1.8.0
+
+    :attr:`background_color` is an :class:`~kivy.properties.AliasProperty`
+    for colors['background'].
     '''
 
     colors = DictProperty({
-        'background': 'e5e6e9',
-        'link': 'ce5c00',
-        'paragraph': '202020',
-        'title': '204a87',
-        'bullet': '000000'})
+        'background': 'e5e6e9ff',
+        'link': 'ce5c00ff',
+        'paragraph': '202020ff',
+        'title': '204a87ff',
+        'bullet': '000000ff'})
     '''Dictionary of all the colors used in the RST rendering.
 
     .. warning::
@@ -423,25 +485,35 @@ class RstDocument(ScrollView):
         This dictionary is needs special handling. You also need to call
         :meth:`RstDocument.render` if you change them after loading.
 
-    :data:`colors` is a :class:`~kivy.properties.DictProperty`.
+    :attr:`colors` is a :class:`~kivy.properties.DictProperty`.
     '''
 
     title = StringProperty('')
     '''Title of the current document.
 
-    :data:`title` is a :class:`~kivy.properties.StringProperty`, default to ''
-    in read-only.
+    :attr:`title` is a :class:`~kivy.properties.StringProperty` and defaults to
+    ''. It is read-only.
     '''
 
     toctrees = DictProperty({})
     '''Toctree of all loaded or preloaded documents. This dictionary is filled
-    when a rst document is explicitly loaded, or where :meth:`preload` has been
+    when a rst document is explicitly loaded or where :meth:`preload` has been
     called.
 
-    If the document has no filename, e.g., when the document is loaded from a
+    If the document has no filename, e.g. when the document is loaded from a
     text file, the key will be ''.
 
-    :data:`toctrees` is a :class:`~kivy.properties.DictProperty`, default to {}.
+    :attr:`toctrees` is a :class:`~kivy.properties.DictProperty` and defaults
+    to {}.
+    '''
+
+    underline_color = StringProperty('204a9699')
+    '''underline color of the titles, expressed in html color notation
+
+    :attr:`underline_color` is a
+    :class:`~kivy.properties.StringProperty` and defaults to '204a9699'.
+
+    .. versionadded: 1.9.0
     '''
 
     # internals.
@@ -454,13 +526,16 @@ class RstDocument(ScrollView):
         self._trigger_load = Clock.create_trigger(self._load_from_text, -1)
         self._parser = rst.Parser()
         self._settings = frontend.OptionParser(
-                components=(rst.Parser, )).get_default_values()
+            components=(rst.Parser, )).get_default_values()
         super(RstDocument, self).__init__(**kwargs)
 
     def on_source(self, instance, value):
+        if not value:
+            return
         if self.document_root is None:
-            # set the documentation root to the directory name of the first tile
-            self.document_root = dirname(value)
+            # set the documentation root to the directory name of the
+            # first tile
+            self.document_root = abspath(dirname(value))
         self._load_from_source()
 
     def on_text(self, instance, value):
@@ -473,25 +548,21 @@ class RstDocument(ScrollView):
 
     def resolve_path(self, filename):
         '''Get the path for this filename. If the filename doesn't exist,
-        it return the document_root + filename.
+        it returns the document_root + filename.
         '''
         if exists(filename):
             return filename
         return join(self.document_root, filename)
 
-    def preload(self, filename):
-        '''Preload a rst file to get its toctree, and its title.
+    def preload(self, filename, encoding='utf-8', errors='strict'):
+        '''Preload a rst file to get its toctree and its title.
 
-        The result will be stored in :data:`toctrees` with the ``filename`` as
+        The result will be stored in :attr:`toctrees` with the ``filename`` as
         key.
         '''
-        if filename in self.toctrees:
-            return
-        if not exists(filename):
-            return
 
-        with open(filename) as fd:
-            text = fd.read()
+        with open(filename, 'rb') as fd:
+            text = fd.read().decode(encoding, errors)
         # parse the source
         document = utils.new_document('Document', self._settings)
         self._parser.parse(text, document)
@@ -499,12 +570,13 @@ class RstDocument(ScrollView):
         visitor = _ToctreeVisitor(document)
         document.walkabout(visitor)
         self.toctrees[filename] = visitor.toctree
+        return text
 
     def _load_from_source(self):
         filename = self.resolve_path(self.source)
-        self.preload(filename)
-        with open(filename) as fd:
-            self.text = fd.read()
+        self.text = self.preload(filename,
+                                 self.source_encoding,
+                                 self.source_error)
 
     def _load_from_text(self, *largs):
         try:
@@ -515,7 +587,10 @@ class RstDocument(ScrollView):
 
             # parse the source
             document = utils.new_document('Document', self._settings)
-            self._parser.parse(self.text, document)
+            text = self.text
+            if PY2 and type(text) is str:
+                text = text.decode('utf-8')
+            self._parser.parse(text, document)
 
             # fill the current document node
             visitor = _Visitor(self, document)
@@ -548,20 +623,21 @@ class RstDocument(ScrollView):
         .. note::
 
             It is preferable to delay the call of the goto if you just loaded
-            the document, because the layout might not be finished, or if the
-            size of the RstDocument is not fixed yet, then the calculation of
-            the scrolling would be wrong.
+            the document because the layout might not be finished or the
+            size of the RstDocument has not yet been determined. In
+            either case, the calculation of the scrolling would be
+            wrong.
 
-            However, you can do a direct call if the document is already loaded.
+            You can, however, do a direct call if the document is already
+            loaded.
 
         .. versionadded:: 1.3.0
         '''
         # check if it's a file ?
-        if self.document_root is not None and ref.endswith('.rst'):
-            filename = join(self.document_root, ref)
-            if exists(filename):
-                self.source = filename
-                return
+        if ref.endswith('.rst'):
+            # whether it's a valid or invalid file, let source deal with it
+            self.source = ref
+            return
 
         # get the association
         ref = self.refs_assoc.get(ref, ref)
@@ -582,11 +658,11 @@ class RstDocument(ScrollView):
         # get the anchor coordinate inside widget space
         ax += node.x
         ay = node.top - ay
-        #ay += node.y
+        # ay += node.y
 
         # what's the current coordinate for us?
         sx, sy = self.scatter.x, self.scatter.top
-        #ax, ay = self.scatter.to_parent(ax, ay)
+        # ax, ay = self.scatter.to_parent(ax, ay)
 
         ay -= self.height
 
@@ -602,6 +678,8 @@ class RstTitle(Label):
 
     section = NumericProperty(0)
 
+    document = ObjectProperty(None)
+
 
 class RstParagraph(Label):
 
@@ -609,10 +687,15 @@ class RstParagraph(Label):
 
     my = NumericProperty(10)
 
+    document = ObjectProperty(None)
+
 
 class RstTerm(AnchorLayout):
+
     text = StringProperty('')
 
+    document = ObjectProperty(None)
+
 
 class RstBlockQuote(GridLayout):
     content = ObjectProperty(None)
@@ -631,7 +714,8 @@ class RstListItem(GridLayout):
 
 
 class RstListBullet(Label):
-    pass
+
+    document = ObjectProperty(None)
 
 
 class RstSystemMessage(GridLayout):
@@ -655,11 +739,13 @@ class RstAsyncImage(AsyncImage):
 
 
 class RstDefinitionList(GridLayout):
-    pass
+
+    document = ObjectProperty(None)
 
 
 class RstDefinition(GridLayout):
-    pass
+
+    document = ObjectProperty(None)
 
 
 class RstFieldList(GridLayout):
@@ -667,7 +753,8 @@ class RstFieldList(GridLayout):
 
 
 class RstFieldName(Label):
-    pass
+
+    document = ObjectProperty(None)
 
 
 class RstFieldBody(GridLayout):
@@ -695,7 +782,8 @@ class RstEmptySpace(Widget):
 
 
 class RstDefinitionSpace(Widget):
-    pass
+
+    document = ObjectProperty(None)
 
 
 class _ToctreeVisitor(nodes.NodeVisitor):
@@ -715,7 +803,6 @@ class _ToctreeVisitor(nodes.NodeVisitor):
 
     def dispatch_visit(self, node):
         cls = node.__class__
-        #print '>>>', cls, node.attlist() if hasattr(node, 'attlist') else ''
         if cls is nodes.section:
             section = {
                 'ids': node['ids'],
@@ -734,7 +821,6 @@ class _ToctreeVisitor(nodes.NodeVisitor):
 
     def dispatch_departure(self, node):
         cls = node.__class__
-        #print '<--', cls, node.attlist() if hasattr(node, 'attlist') else ''
         if cls is nodes.section:
             self.pop()
         elif cls is nodes.title:
@@ -764,7 +850,6 @@ class _Visitor(nodes.NodeVisitor):
 
     def dispatch_visit(self, node):
         cls = node.__class__
-        #print '>>>', cls, node.attlist() if hasattr(node, 'attlist') else ''
         if cls is nodes.document:
             self.push(self.root.content)
 
@@ -772,10 +857,10 @@ class _Visitor(nodes.NodeVisitor):
             self.section += 1
 
         elif cls is nodes.title:
-            label = RstTitle(section=self.section)
+            label = RstTitle(section=self.section, document=self.root)
             self.current.add_widget(label)
             self.push(label)
-            #assert(self.text == '')
+            # assert(self.text == '')
 
         elif cls is nodes.Text:
             if self.do_strip_text:
@@ -793,7 +878,7 @@ class _Visitor(nodes.NodeVisitor):
 
         elif cls is nodes.paragraph:
             self.do_strip_text = True
-            label = RstParagraph()
+            label = RstParagraph(document=self.root)
             if isinstance(self.current, RstEntry):
                 label.mx = 10
             self.current.add_widget(label)
@@ -811,7 +896,7 @@ class _Visitor(nodes.NodeVisitor):
             self.text += '[b]'
 
         elif cls is nodes.literal:
-            self.text += '[font=fonts/DroidSansMono.ttf]'
+            self.text += '[font=fonts/RobotoMono-Regular.ttf]'
 
         elif cls is nodes.block_quote:
             box = RstBlockQuote()
@@ -838,7 +923,8 @@ class _Visitor(nodes.NodeVisitor):
                 bullet = '%d.' % self.idx_list
             bullet = self.colorize(bullet, 'bullet')
             item = RstListItem()
-            self.current.add_widget(RstListBullet(text=bullet))
+            self.current.add_widget(RstListBullet(
+                text=bullet, document=self.root))
             self.current.add_widget(item)
             self.push(item)
 
@@ -870,26 +956,27 @@ class _Visitor(nodes.NodeVisitor):
                 image = RstImage(source=uri)
 
             align = node.get('align', 'center')
-            root = AnchorLayout(size_hint_y=None, anchor_x=align, height=1)
+            root = AnchorLayout(size_hint_y=None, anchor_x=align,
+                                height=image.height)
             image.bind(height=root.setter('height'))
             root.add_widget(image)
             self.current.add_widget(root)
 
         elif cls is nodes.definition_list:
-            lst = RstDefinitionList()
+            lst = RstDefinitionList(document=self.root)
             self.current.add_widget(lst)
             self.push(lst)
 
         elif cls is nodes.term:
             assert(isinstance(self.current, RstDefinitionList))
-            term = RstTerm()
+            term = RstTerm(document=self.root)
             self.current.add_widget(term)
             self.push(term)
 
         elif cls is nodes.definition:
             assert(isinstance(self.current, RstDefinitionList))
-            definition = RstDefinition()
-            definition.add_widget(RstDefinitionSpace())
+            definition = RstDefinition(document=self.root)
+            definition.add_widget(RstDefinitionSpace(document=self.root))
             self.current.add_widget(definition)
             self.push(definition)
 
@@ -899,7 +986,7 @@ class _Visitor(nodes.NodeVisitor):
             self.push(fieldlist)
 
         elif cls is nodes.field_name:
-            name = RstFieldName()
+            name = RstFieldName(document=self.root)
             self.current.add_widget(name)
             self.push(name)
 
@@ -926,9 +1013,9 @@ class _Visitor(nodes.NodeVisitor):
 
         elif cls is nodes.reference:
             name = node.get('name', node.get('refuri'))
-            self.text += '[ref=%s][color=%s]' % (name,
-                    self.root.colors.get('link',
-                        self.root.colors.get('paragraph')))
+            self.text += '[ref=%s][color=%s]' % (
+                name, self.root.colors.get(
+                    'link', self.root.colors.get('paragraph')))
             if 'refname' in node and 'name' in node:
                 self.root.refs_assoc[node['name']] = node['refname']
 
@@ -949,7 +1036,6 @@ class _Visitor(nodes.NodeVisitor):
 
     def dispatch_departure(self, node):
         cls = node.__class__
-        #print '<--', cls
         if cls is nodes.document:
             self.pop()
 
@@ -1054,7 +1140,8 @@ class _Visitor(nodes.NodeVisitor):
             filename = self.root.resolve_path(rst_docname)
             self.root.preload(filename)
 
-            # if exist, use the title of the first section found in the document
+            # if exist, use the title of the first section found in the
+            # document
             title = docname
             if filename in self.root.toctrees:
                 toctree = self.root.toctrees[filename]
@@ -1063,8 +1150,8 @@ class _Visitor(nodes.NodeVisitor):
 
             # replace the text with a good reference
             text = '[ref=%s]%s[/ref]' % (
-                    rst_docname,
-                    self.colorize(title, 'link'))
+                rst_docname,
+                self.colorize(title, 'link'))
             self.text = self.text[:self.doc_index] + text
 
         elif cls is role_video:
@@ -1074,9 +1161,9 @@ class _Visitor(nodes.NodeVisitor):
             if uri.startswith('/') and self.root.document_root:
                 uri = join(self.root.document_root, uri[1:])
             video = RstVideoPlayer(
-                    source=uri,
-                    size_hint=(None, None),
-                    size=(width, height))
+                source=uri,
+                size_hint=(None, None),
+                size=(width, height))
             anchor = AnchorLayout(size_hint_y=None, height=height + 20)
             anchor.add_widget(video)
             self.current.add_widget(anchor)
@@ -1098,6 +1185,7 @@ class _Visitor(nodes.NodeVisitor):
             self.root.colors.get(name, self.root.colors['paragraph']),
             text)
 
+
 if __name__ == '__main__':
     from kivy.base import runTouchApp
     import sys
